package io.zipcoder;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ItemParser {

    public ArrayList<String> parseRawDataIntoStringArray(String rawData){
        String stringPattern = "##";
        ArrayList<String> response = splitStringWithRegexPattern(stringPattern , rawData);
        return response;
    }

    public Item parseStringIntoItem(String rawItem) throws ItemParseException{
        ArrayList<String> values = getValues(rawItem);

        String name = uppercase(correctSpelling(values.get(0)));
        Double price = Double.valueOf(values.get(1));
        String type = uppercase(values.get(2));
        String expiration = values.get(3);

        return new Item(name, price, type, expiration);
    }

    public ArrayList<String> findKeyValuePairsInRawItemData(String rawItem){
        String stringPattern = "[^a-zA-Z0-9\\:\\.\\/]";
        ArrayList<String> response = splitStringWithRegexPattern(stringPattern , rawItem);
        return response;
    }

    private ArrayList<String> splitStringWithRegexPattern(String stringPattern, String inputString){
        return new ArrayList<String>(Arrays.asList(inputString.split(stringPattern)));
    }

    //keys - (.+)(?=:)
    //values - (?<=:)(.+)

    public String getValue(String input)throws ItemParseException{
        ArrayList<String> kv = splitStringWithRegexPattern(":", input);
        if(kv.size() < 2){
            throw new ItemParseException();
        } else if(kv.size() == 2) {
            return kv.get(1);
        }
        return null;
    }

    public ArrayList<String> getValues(String rawItem) throws ItemParseException{
        ArrayList<String> KVPairs = findKeyValuePairsInRawItemData(rawItem);
        ArrayList<String> values = new ArrayList<String>();
        for(String pair : KVPairs){
            String value = toLowerCase(getValue(pair));
            if(value.equals("")){
                throw new ItemParseException();
            } else{
                values.add(value);
            }
        }
        return values;
    }

    public String toLowerCase(String input) {
        String[] lower = {"a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z"};
        String[] upper = {"A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z"};

        StringBuilder lowercase = new StringBuilder();

        Pattern pattern = Pattern.compile(".");
        Matcher matcher = pattern.matcher(input);

        while(matcher.find()){
            String character = matcher.group();
            if(isUpper(character)){
                int i = indexOf(upper, character);
                lowercase.append(lower[i]);
            }else {
                lowercase.append(character);
            }
        }
        return lowercase.toString();
    }

    public boolean isUpper(String c){
        Pattern pattern = Pattern.compile("[A-Z]");
        Matcher matcher = pattern.matcher(c);
        return matcher.find();
    }

    public int indexOf(String[] array, String c){
        for(int i = 0; i < array.length; i++){
            if(array[i].equals(c)){
                return i;
            }
        }
        return -1;
    }














    public int findNumberOfErrors(String input){
        ArrayList<String> badFormat = new ArrayList<String>();
        ArrayList<String> rawData = parseRawDataIntoStringArray(input);
        for(String data : rawData){
            try {
                Item item = parseStringIntoItem(data);
            }catch(ItemParseException e){
                badFormat.add(data);
            }
        }
        return badFormat.size();
    }

    public ArrayList<Item> findItems(String input){
        ArrayList<Item> items = new ArrayList<Item>();
        ArrayList<String> rawData = parseRawDataIntoStringArray(input);
        for(String data : rawData){
            try {
                Item item = parseStringIntoItem(data);
                items.add(item);
            }catch(ItemParseException e){
                e.printStackTrace();
            }
        }
        return items;
    }

    public Map<Object, Integer> getUniqueItemNames(ArrayList<Item> items, String methodname){

        try {
            Map<Object, Integer> uniqueNames = new HashMap<>();
            for (Item item : items) {
                Class<?> e = item.getClass();
                Method method = e.getMethod(methodname);
                if (!uniqueNames.containsKey(method.invoke(item))) {
                    uniqueNames.put(method.invoke(item), new Integer(1));
                } else {
                    int amount = uniqueNames.get(method.invoke(item));
                    amount = amount + 1;
                    uniqueNames.put(method.invoke(item), amount);
                }
            }
            return uniqueNames;

        }catch(Exception e){
            e.printStackTrace();
        }
        return null;
    }

    public String correctSpelling(String test) {
        String[] nums = {"0", "1", "3"};
        String[] alpha = {"o", "l", "e"};

        StringBuilder lowercase = new StringBuilder();

        Pattern pattern = Pattern.compile(".");
        Matcher matcher = pattern.matcher(test);

        while(matcher.find()){
            String character = matcher.group();
            int index = indexOf(nums, character);
            if(index != -1){
                lowercase.append(alpha[index]);
            } else{
                lowercase.append(character);
            }
        }
        return lowercase.toString();
    }

    public Map<String, Map<Double, Integer>> sortItems(ArrayList<Item> items){
        Map<String, Map<Double, Integer>> uniqueItem = new HashMap<>();
        Map<Object, Integer> names = getUniqueItemNames(items, "getName");

        for(Object key : names.keySet()){
            Map<Double, Integer> prices = new HashMap<>();
            for(Item item : items){
                if(item.getName().equals(key)){
                    if(prices.containsKey(item.getPrice())){
                        int amount = prices.get(item.getPrice());
                        amount = amount + 1;
                        prices.put(item.getPrice(), amount);
                    } else {
                        prices.put(item.getPrice(), 1);
                    }
                }
                uniqueItem.put((String)key, prices);
            }
        }

        return uniqueItem;
    }

    public String uppercase(String test) {
        String[] lower = {"a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z"};
        String[] upper = {"A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z"};

        ArrayList<String> answer = getCharList(test);

        if(answer.size() > 0){
            if(isUpper(answer.get(0))){
                return listToString(answer);
            } else {
                int index = indexOf(lower, answer.get(0));
                answer.remove(0);
                answer.add(0, upper[index]);
            }
        }
        return listToString(answer);
    }

    public ArrayList<String> getCharList(String input){
        ArrayList<String> answer = new ArrayList<>();

        Pattern pattern = Pattern.compile(".");
        Matcher matcher = pattern.matcher(input);

        while(matcher.find()){
            answer.add(matcher.group());
        }
        return answer;
    }

    public String listToString(List<String> list){
        StringBuilder stringBuilder = new StringBuilder();
        for(String string : list){
            stringBuilder.append(string);
        }
        return stringBuilder.toString();
    }

    public void print(String input){
        int errs = findNumberOfErrors(input);
        ArrayList<Item> items = findItems(input);
        Map<Object, Integer> names = getUniqueItemNames(items, "getName");
        Map<String, Map<Double, Integer>> uniqueItem = sortItems(items);

        for(String name : uniqueItem.keySet()){
            System.out.printf("name: %7s %20s \n", name, "seen: " + names.get(name) + " times");
            System.out.println("=============     \t =============");
            for(Double price : uniqueItem.get(name).keySet()){
                System.out.printf("Price: %6s %20s \n", price, "seen: " + uniqueItem.get(name).get(price) + " times");
            }
            System.out.println("=============     \t =============");

        }
        System.out.printf("\nErrors %26s", "seen " + errs + " times" );
    }
}
